import 'dart:core';

import 'package:fis_jsonrpc/client_base.dart';
import 'package:fis_common/json_convert.dart';

import 'authentication.m.dart';


class AuthenticationService extends JsonRpcClientBase {
	AuthenticationService(
		String host, {
		String serviceName = "IAuthenticationService",
		Map<String, String>? headers,
		int? timeout,
	}) : super(
						host,
						serviceName,
						headers: headers,
						timeout: timeout,
				) {
		/// 注册响应实体反序列化处理器
		FJsonConvert.setDecoder((map) => Token.fromJson(map));
		FJsonConvert.setDecoder((map) => ValidateTokenResult.fromJson(map));
		FJsonConvert.setDecoder((map) => IList<Token>.fromJson(map));
	}

	Future<String> getAuthorizationAsync(AuthenticationRequest request) async {
		var rpcRst = await call("GetAuthorizationAsync", request);
		return rpcRst;
	}

	Future<String> getServerDefaultTokenAsync() async {
		var rpcRst = await call("GetServerDefaultTokenAsync", );
		return rpcRst;
	}

	Future<Token> applyTokenAsync(ApplyTokenRequest request) async {
		var rpcRst = await call("ApplyTokenAsync", request);
		var result = Token.fromJson(rpcRst as Map<String, dynamic>);
		return result;
	}

	Future<ValidateTokenResult> validateTokenAsync(ValidateTokenRequest request) async {
		var rpcRst = await call("ValidateTokenAsync", request);
		var result = ValidateTokenResult.fromJson(rpcRst as Map<String, dynamic>);
		return result;
	}

	Future<IList<Token>> getTokensWithClientIdAsync(GetTokensWithClientIdRequest request) async {
		var rpcRst = await call("GetTokensWithClientIdAsync", request);
		var result = IList<Token>.fromJson(rpcRst as Map<String, dynamic>);
		return result;
	}

	Future<bool> logOffAsync(TokenRequest request) async {
		var rpcRst = await call("LogOffAsync", request);
		return rpcRst;
	}

}

