import 'package:fis_jsonrpc/utils.dart';

class BaseRequest {

	BaseRequest();

	factory BaseRequest.fromJson(Map<String, dynamic> map) {
		return BaseRequest( 
		);
	}

	Map<String, dynamic> toJson() {
		final map = Map<String, dynamic>();
		return map;
	}
}

class TokenRequest extends BaseRequest{
	String? token;

	TokenRequest({
		this.token,
	}) : super(
		);

	factory TokenRequest.fromJson(Map<String, dynamic> map) {
		return TokenRequest( 
			token: map['Token'],
		);
	}

	Map<String, dynamic> toJson() {
		final map = super.toJson();
		if(token != null)
			map['Token'] = token;
		return map;
	}
}

class AuthenticationRequest extends TokenRequest{
	String? fileName;

	AuthenticationRequest({
		this.fileName,
		String? token,
	}) : super(
			token: token,
		);

	factory AuthenticationRequest.fromJson(Map<String, dynamic> map) {
		return AuthenticationRequest( 
			fileName: map['FileName'],
			token: map['Token'],
		);
	}

	Map<String, dynamic> toJson() {
		final map = super.toJson();
		if(fileName != null)
			map['FileName'] = fileName;
		return map;
	}
}

enum AccountType {
	Admin,
	User,
	US,
	USBox,
}

enum Platform {
	Windows,
	Android,
	Ios,
}

enum LoginSource {
	PC,
	Mobile,
	Pad,
	Web,
}

class Token {
	int version;
	String? value;
	AccountType accountType;
	Platform platform;
	LoginSource loginSource;
	String? clientId;
	String? loginServer;
	DateTime? createTime;
	DateTime? expiration;
	int ipValue;

	Token({
		this.version = 0,
		this.value,
		this.accountType = AccountType.Admin,
		this.platform = Platform.Windows,
		this.loginSource = LoginSource.PC,
		this.clientId,
		this.loginServer,
		this.createTime,
		this.expiration,
		this.ipValue = 0,
	});

	factory Token.fromJson(Map<String, dynamic> map) {
		return Token( 
			version: map['Version'],
			value: map['Value'],
			accountType: AccountType.values.firstWhere((e) => e.index == map['AccountType']),
			platform: Platform.values.firstWhere((e) => e.index == map['Platform']),
			loginSource: LoginSource.values.firstWhere((e) => e.index == map['LoginSource']),
			clientId: map['ClientId'],
			loginServer: map['LoginServer'],
			createTime: map['CreateTime'] != null ? DateTime.parse(map['CreateTime']) : null,
			expiration: map['Expiration'] != null ? DateTime.parse(map['Expiration']) : null,
			ipValue: map['IpValue'],
		);
	}

	Map<String, dynamic> toJson() {
		final map = Map<String, dynamic>();
		map['Version'] = version;
		if(value != null)
			map['Value'] = value;
		map['AccountType'] = accountType.index;
		map['Platform'] = platform.index;
		map['LoginSource'] = loginSource.index;
		if(clientId != null)
			map['ClientId'] = clientId;
		if(loginServer != null)
			map['LoginServer'] = loginServer;
		if(createTime != null)
			map['CreateTime'] = JsonRpcUtils.dateFormat(createTime!);
		if(expiration != null)
			map['Expiration'] = JsonRpcUtils.dateFormat(expiration!);
		map['IpValue'] = ipValue;
		return map;
	}
}

class ApplyTokenRequest {
	AccountType accountType;
	Platform platform;
	LoginSource loginSource;
	String? clientId;
	String? loginServer;
	int ipValue;

	ApplyTokenRequest({
		this.accountType = AccountType.Admin,
		this.platform = Platform.Windows,
		this.loginSource = LoginSource.PC,
		this.clientId,
		this.loginServer,
		this.ipValue = 0,
	});

	factory ApplyTokenRequest.fromJson(Map<String, dynamic> map) {
		return ApplyTokenRequest( 
			accountType: AccountType.values.firstWhere((e) => e.index == map['AccountType']),
			platform: Platform.values.firstWhere((e) => e.index == map['Platform']),
			loginSource: LoginSource.values.firstWhere((e) => e.index == map['LoginSource']),
			clientId: map['ClientId'],
			loginServer: map['LoginServer'],
			ipValue: map['IpValue'],
		);
	}

	Map<String, dynamic> toJson() {
		final map = Map<String, dynamic>();
		map['AccountType'] = accountType.index;
		map['Platform'] = platform.index;
		map['LoginSource'] = loginSource.index;
		if(clientId != null)
			map['ClientId'] = clientId;
		if(loginServer != null)
			map['LoginServer'] = loginServer;
		map['IpValue'] = ipValue;
		return map;
	}
}

enum CustomerRpcCode {
	Ok,
	TokenNotExist,
	TokenExpired,
	InvalidTokenVersion,
	IPInBlacklist,
}

class ValidateTokenResult {
	CustomerRpcCode code;
	Token? token;

	ValidateTokenResult({
		this.code = CustomerRpcCode.Ok,
		this.token,
	});

	factory ValidateTokenResult.fromJson(Map<String, dynamic> map) {
		return ValidateTokenResult( 
			code: CustomerRpcCode.values.firstWhere((e) => e.index == map['Code']),
			token: map['Token'] != null ? Token.fromJson(map['Token']) : null,
		);
	}

	Map<String, dynamic> toJson() {
		final map = Map<String, dynamic>();
		map['Code'] = code.index;
		if(token != null)
			map['Token'] = token;
		return map;
	}
}

class ValidateTokenRequest {
	String? token;

	ValidateTokenRequest({
		this.token,
	});

	factory ValidateTokenRequest.fromJson(Map<String, dynamic> map) {
		return ValidateTokenRequest( 
			token: map['Token'],
		);
	}

	Map<String, dynamic> toJson() {
		final map = Map<String, dynamic>();
		if(token != null)
			map['Token'] = token;
		return map;
	}
}

class IList<T> {
	Token? item;

	IList({
		this.item,
	});

	factory IList.fromJson(Map<String, dynamic> map) {
		return IList( 
			item: map['Item'] != null ? Token.fromJson(map['Item']) : null,
		);
	}

	Map<String, dynamic> toJson() {
		final map = Map<String, dynamic>();
		if(item != null)
			map['Item'] = item;
		return map;
	}
}

class GetTokensWithClientIdRequest extends BaseRequest{
	String? clientId;
	AccountType accountType;

	GetTokensWithClientIdRequest({
		this.clientId,
		this.accountType = AccountType.Admin,
	}) : super(
		);

	factory GetTokensWithClientIdRequest.fromJson(Map<String, dynamic> map) {
		return GetTokensWithClientIdRequest( 
			clientId: map['ClientId'],
			accountType: AccountType.values.firstWhere((e) => e.index == map['AccountType']),
		);
	}

	Map<String, dynamic> toJson() {
		final map = super.toJson();
		if(clientId != null)
			map['ClientId'] = clientId;
		map['AccountType'] = accountType.index;
		return map;
	}
}


