import 'dart:core';

import 'package:fis_jsonrpc/client_base.dart';
import 'package:fis_common/json_convert.dart';

import 'remedical.m.dart';

import 'recordInfo.m.dart';
import 'organization.m.dart';
import 'patient.m.dart';


class RemedicalService extends JsonRpcClientBase {
	RemedicalService(
		String host, {
		String serviceName = "IRemedicalService",
		Map<String, String>? headers,
		int? timeout,
	}) : super(
						host,
						serviceName,
						headers: headers,
						timeout: timeout,
				) {
		/// 注册响应实体反序列化处理器
		FJsonConvert.setDecoder((map) => CreateExaminfoResult.fromJson(map));
		FJsonConvert.setDecoder((map) => RemedicalInfoDTO.fromJson(map));
		FJsonConvert.setDecoder((map) => RemedicalListResult.fromJson(map));
		FJsonConvert.setDecoder((map) => PageResult<RemedicalListResult>.fromJson(map));
		FJsonConvert.setDecoder((map) => DataItemDTO.fromJson(map));
		FJsonConvert.setDecoder((map) => OrganizationSettingResult.fromJson(map));
		FJsonConvert.setDecoder((map) => QueryReservationResult.fromJson(map));
		FJsonConvert.setDecoder((map) => PageResult<QueryExamListItemResult>.fromJson(map));
		FJsonConvert.setDecoder((map) => QueryExamListItemResult.fromJson(map));
		FJsonConvert.setDecoder((map) => QueryExamInfoResult.fromJson(map));
		FJsonConvert.setDecoder((map) => MeasureApplicationDTO.fromJson(map));
		FJsonConvert.setDecoder((map) => CommentItemDTO.fromJson(map));
		FJsonConvert.setDecoder((map) => PresetCommentItemDTO.fromJson(map));
	}

	Future<CreateExaminfoResult> createExamInfoAsync(CreateExaminfoRequest request) async {
		var rpcRst = await call("CreateExamInfoAsync", request);
		var result = CreateExaminfoResult.fromJson(rpcRst as Map<String, dynamic>);
		return result;
	}

	Future<bool> uploadExamDataAsync(UploadExamDataRequest request) async {
		var rpcRst = await call("UploadExamDataAsync", request);
		return rpcRst;
	}

	Future<RemedicalInfoDTO> findRemedicalByCodeAsync(FindRemedicalByCodeRequest request) async {
		var rpcRst = await call("FindRemedicalByCodeAsync", request);
		var result = RemedicalInfoDTO.fromJson(rpcRst as Map<String, dynamic>);
		return result;
	}

	Future<bool> addToRemedicalDiagnosisAsync(AddToRemedicalDiagnosisRequest request) async {
		var rpcRst = await call("AddToRemedicalDiagnosisAsync", request);
		return rpcRst;
	}

	Future<RemedicalListResult> getRemedicalListByRecordInfoAsync(QueryRecordRequest request) async {
		var rpcRst = await call("GetRemedicalListByRecordInfoAsync", request);
		var result = RemedicalListResult.fromJson(rpcRst as Map<String, dynamic>);
		return result;
	}

	Future<PageResult<RemedicalListResult>> getRemedicalListPagesAsync(GetRecordsPageRequest request) async {
		var rpcRst = await call("GetRemedicalListPagesAsync", request);
		var result = PageResult<RemedicalListResult>.fromJson(rpcRst as Map<String, dynamic>);
		return result;
	}

	Future<List<DataItemDTO>> queryDropdownListAsync(QueryDropdownListReuqest request) async {
		var rpcRst = await call("QueryDropdownListAsync", request);
		var result = (rpcRst as List).map((e)=>DataItemDTO.fromJson(e as Map<String, dynamic>)).toList();
		return result;
	}

	Future<OrganizationSettingResult> getOrganizationSettingAsync(GetUserOrganizationSettingRequest request) async {
		var rpcRst = await call("GetOrganizationSettingAsync", request);
		var result = OrganizationSettingResult.fromJson(rpcRst as Map<String, dynamic>);
		return result;
	}

	Future<QueryReservationResult> queryReservationAsync(QueryReservationRequest request) async {
		var rpcRst = await call("QueryReservationAsync", request);
		var result = QueryReservationResult.fromJson(rpcRst as Map<String, dynamic>);
		return result;
	}

	Future<bool> pushFinshExamNotificationToClientAsync(PushFinishExamNotifyToClientRequest request) async {
		var rpcRst = await call("PushFinshExamNotificationToClientAsync", request);
		return rpcRst;
	}

	Future<bool> deviceFinishExamAsync(DeviceFinishExamRequest request) async {
		var rpcRst = await call("DeviceFinishExamAsync", request);
		return rpcRst;
	}

	Future<PageResult<QueryExamListItemResult>> queryExamListAsync(QueryExamListRequest request) async {
		var rpcRst = await call("QueryExamListAsync", request);
		var result = PageResult<QueryExamListItemResult>.fromJson(rpcRst as Map<String, dynamic>);
		return result;
	}

	Future<QueryExamInfoResult> queryExamInfoAsync(QueryExamInfoRequest request) async {
		var rpcRst = await call("QueryExamInfoAsync", request);
		var result = QueryExamInfoResult.fromJson(rpcRst as Map<String, dynamic>);
		return result;
	}

	Future<MeasureApplicationDTO> getMeasureApplicationAsync(GetMeasureApplicationRequest request) async {
		var rpcRst = await call("GetMeasureApplicationAsync", request);
		var result = MeasureApplicationDTO.fromJson(rpcRst as Map<String, dynamic>);
		return result;
	}

	Future<List<CommentItemDTO>> getCommentsByApplicationAsync(GetCommentsByApplicationRequest request) async {
		var rpcRst = await call("GetCommentsByApplicationAsync", request);
		var result = (rpcRst as List).map((e)=>CommentItemDTO.fromJson(e as Map<String, dynamic>)).toList();
		return result;
	}

	Future<List<PresetCommentItemDTO>> getPresetCommentsAsync(GetPresetCommentsRequest request) async {
		var rpcRst = await call("GetPresetCommentsAsync", request);
		var result = (rpcRst as List).map((e)=>PresetCommentItemDTO.fromJson(e as Map<String, dynamic>)).toList();
		return result;
	}

	Future<bool> deviceDiagnosisChanged(DeviceDiagnosisChangedNotification request) async {
		var rpcRst = await call("DeviceDiagnosisChanged", request);
		return rpcRst;
	}

	Future<bool> clearDeviceDiagnosis() async {
		var rpcRst = await call("ClearDeviceDiagnosis", );
		return rpcRst;
	}

}

