import 'authentication.m.dart';
import 'patient.m.dart';
import 'recordInfo.m.dart';
import 'connect.m.dart';
import 'device.m.dart';
import 'organization.m.dart';

import 'package:fis_jsonrpc/utils.dart';

class CreateExaminfoResult {
	String? examCode;

	CreateExaminfoResult({
		this.examCode,
	});

	factory CreateExaminfoResult.fromJson(Map<String, dynamic> map) {
		return CreateExaminfoResult( 
			examCode: map['ExamCode'],
		);
	}

	Map<String, dynamic> toJson() {
		final map = Map<String, dynamic>();
		if(examCode != null)
			map['ExamCode'] = examCode;
		return map;
	}
}

class CreateExaminfoRequest extends TokenRequest{
	String? patientType;
	String? reservationCode;
	List<DataItemDTO>? patientInfo;
	List<PatientInfoExt>? patientScanInfoList;

	CreateExaminfoRequest({
		this.patientType,
		this.reservationCode,
		this.patientInfo,
		this.patientScanInfoList,
		String? token,
	}) : super(
			token: token,
		);

	factory CreateExaminfoRequest.fromJson(Map<String, dynamic> map) {
		return CreateExaminfoRequest( 
			patientType: map['PatientType'],
			reservationCode: map['ReservationCode'],
			patientInfo: map['PatientInfo'] != null ? (map['PatientInfo'] as List).map((e)=>DataItemDTO.fromJson(e as Map<String,dynamic>)).toList() : null,
			patientScanInfoList: map['PatientScanInfoList'] != null ? (map['PatientScanInfoList'] as List).map((e)=>PatientInfoExt.fromJson(e as Map<String,dynamic>)).toList() : null,
			token: map['Token'],
		);
	}

	Map<String, dynamic> toJson() {
		final map = super.toJson();
		if(patientType != null)
			map['PatientType'] = patientType;
		if(reservationCode != null)
			map['ReservationCode'] = reservationCode;
		if(patientInfo != null)
			map['PatientInfo'] = patientInfo;
		if(patientScanInfoList != null)
			map['PatientScanInfoList'] = patientScanInfoList;
		return map;
	}
}

class UploadExamDataRequest extends TokenRequest{
	String? examCode;
	String? previewFileToken;
	String? fileToken;
	String? coverImageToken;
	String? application;
	RemedicalFileDataTypeEnum fileDataType;

	UploadExamDataRequest({
		this.examCode,
		this.previewFileToken,
		this.fileToken,
		this.coverImageToken,
		this.application,
		this.fileDataType = RemedicalFileDataTypeEnum.VinnoVidSingle,
		String? token,
	}) : super(
			token: token,
		);

	factory UploadExamDataRequest.fromJson(Map<String, dynamic> map) {
		return UploadExamDataRequest( 
			examCode: map['ExamCode'],
			previewFileToken: map['PreviewFileToken'],
			fileToken: map['FileToken'],
			coverImageToken: map['CoverImageToken'],
			application: map['Application'],
			fileDataType: RemedicalFileDataTypeEnum.values.firstWhere((e) => e.index == map['FileDataType']),
			token: map['Token'],
		);
	}

	Map<String, dynamic> toJson() {
		final map = super.toJson();
		if(examCode != null)
			map['ExamCode'] = examCode;
		if(previewFileToken != null)
			map['PreviewFileToken'] = previewFileToken;
		if(fileToken != null)
			map['FileToken'] = fileToken;
		if(coverImageToken != null)
			map['CoverImageToken'] = coverImageToken;
		if(application != null)
			map['Application'] = application;
		map['FileDataType'] = fileDataType.index;
		return map;
	}
}

class FindRemedicalByCodeRequest extends TokenRequest{
	String? remedicalCode;
	bool existDiagnosisResult;

	FindRemedicalByCodeRequest({
		this.remedicalCode,
		this.existDiagnosisResult = false,
		String? token,
	}) : super(
			token: token,
		);

	factory FindRemedicalByCodeRequest.fromJson(Map<String, dynamic> map) {
		return FindRemedicalByCodeRequest( 
			remedicalCode: map['RemedicalCode'],
			existDiagnosisResult: map['ExistDiagnosisResult'],
			token: map['Token'],
		);
	}

	Map<String, dynamic> toJson() {
		final map = super.toJson();
		if(remedicalCode != null)
			map['RemedicalCode'] = remedicalCode;
		map['ExistDiagnosisResult'] = existDiagnosisResult;
		return map;
	}
}

class AddToRemedicalDiagnosisRequest extends TokenRequest{
	String? remedicalCode;

	AddToRemedicalDiagnosisRequest({
		this.remedicalCode,
		String? token,
	}) : super(
			token: token,
		);

	factory AddToRemedicalDiagnosisRequest.fromJson(Map<String, dynamic> map) {
		return AddToRemedicalDiagnosisRequest( 
			remedicalCode: map['RemedicalCode'],
			token: map['Token'],
		);
	}

	Map<String, dynamic> toJson() {
		final map = super.toJson();
		if(remedicalCode != null)
			map['RemedicalCode'] = remedicalCode;
		return map;
	}
}

class RemedicalItemList {
	String? patientScanTypeDesc;
	List<RemedicalInfoDTO>? remedicalList;

	RemedicalItemList({
		this.patientScanTypeDesc,
		this.remedicalList,
	});

	factory RemedicalItemList.fromJson(Map<String, dynamic> map) {
		return RemedicalItemList( 
			patientScanTypeDesc: map['PatientScanTypeDesc'],
			remedicalList: map['RemedicalList'] != null ? (map['RemedicalList'] as List).map((e)=>RemedicalInfoDTO.fromJson(e as Map<String,dynamic>)).toList() : null,
		);
	}

	Map<String, dynamic> toJson() {
		final map = Map<String, dynamic>();
		if(patientScanTypeDesc != null)
			map['PatientScanTypeDesc'] = patientScanTypeDesc;
		if(remedicalList != null)
			map['RemedicalList'] = remedicalList;
		return map;
	}
}

class RemedicalListResult {
	String? sacnDate;
	String? recordCode;
	List<RemedicalItemList>? remedicalItemList;

	RemedicalListResult({
		this.sacnDate,
		this.recordCode,
		this.remedicalItemList,
	});

	factory RemedicalListResult.fromJson(Map<String, dynamic> map) {
		return RemedicalListResult( 
			sacnDate: map['SacnDate'],
			recordCode: map['RecordCode'],
			remedicalItemList: map['RemedicalItemList'] != null ? (map['RemedicalItemList'] as List).map((e)=>RemedicalItemList.fromJson(e as Map<String,dynamic>)).toList() : null,
		);
	}

	Map<String, dynamic> toJson() {
		final map = Map<String, dynamic>();
		if(sacnDate != null)
			map['SacnDate'] = sacnDate;
		if(recordCode != null)
			map['RecordCode'] = recordCode;
		if(remedicalItemList != null)
			map['RemedicalItemList'] = remedicalItemList;
		return map;
	}
}

enum QueryDropdownListEnum {
	Org,
}

class QueryDropdownListReuqest extends TokenRequest{
	QueryDropdownListEnum queryType;
	String? queryValue;

	QueryDropdownListReuqest({
		this.queryType = QueryDropdownListEnum.Org,
		this.queryValue,
		String? token,
	}) : super(
			token: token,
		);

	factory QueryDropdownListReuqest.fromJson(Map<String, dynamic> map) {
		return QueryDropdownListReuqest( 
			queryType: QueryDropdownListEnum.values.firstWhere((e) => e.index == map['QueryType']),
			queryValue: map['QueryValue'],
			token: map['Token'],
		);
	}

	Map<String, dynamic> toJson() {
		final map = super.toJson();
		map['QueryType'] = queryType.index;
		if(queryValue != null)
			map['QueryValue'] = queryValue;
		return map;
	}
}

class RservationResult {
	String? reservationCode;
	List<DataItemDTO>? patientInfo;
	List<PatientInfoExt>? patientInfoExtList;
	String? dataSource;

	RservationResult({
		this.reservationCode,
		this.patientInfo,
		this.patientInfoExtList,
		this.dataSource,
	});

	factory RservationResult.fromJson(Map<String, dynamic> map) {
		return RservationResult( 
			reservationCode: map['ReservationCode'],
			patientInfo: map['PatientInfo'] != null ? (map['PatientInfo'] as List).map((e)=>DataItemDTO.fromJson(e as Map<String,dynamic>)).toList() : null,
			patientInfoExtList: map['PatientInfoExtList'] != null ? (map['PatientInfoExtList'] as List).map((e)=>PatientInfoExt.fromJson(e as Map<String,dynamic>)).toList() : null,
			dataSource: map['DataSource'],
		);
	}

	Map<String, dynamic> toJson() {
		final map = Map<String, dynamic>();
		if(reservationCode != null)
			map['ReservationCode'] = reservationCode;
		if(patientInfo != null)
			map['PatientInfo'] = patientInfo;
		if(patientInfoExtList != null)
			map['PatientInfoExtList'] = patientInfoExtList;
		if(dataSource != null)
			map['DataSource'] = dataSource;
		return map;
	}
}

class QueryReservationResult {
	List<RservationResult>? reservationList;

	QueryReservationResult({
		this.reservationList,
	});

	factory QueryReservationResult.fromJson(Map<String, dynamic> map) {
		return QueryReservationResult( 
			reservationList: map['ReservationList'] != null ? (map['ReservationList'] as List).map((e)=>RservationResult.fromJson(e as Map<String,dynamic>)).toList() : null,
		);
	}

	Map<String, dynamic> toJson() {
		final map = Map<String, dynamic>();
		if(reservationList != null)
			map['ReservationList'] = reservationList;
		return map;
	}
}

class QueryReservationRequest extends TokenRequest{
	DateTime? createTime;

	QueryReservationRequest({
		this.createTime,
		String? token,
	}) : super(
			token: token,
		);

	factory QueryReservationRequest.fromJson(Map<String, dynamic> map) {
		return QueryReservationRequest( 
			createTime: map['CreateTime'] != null ? DateTime.parse(map['CreateTime']) : null,
			token: map['Token'],
		);
	}

	Map<String, dynamic> toJson() {
		final map = super.toJson();
		if(createTime != null)
			map['CreateTime'] = JsonRpcUtils.dateFormat(createTime!);
		return map;
	}
}

class FinishExamNotifyDetail {
	String? recordCode;
	RecordStatusEnum recordStatus;

	FinishExamNotifyDetail({
		this.recordCode,
		this.recordStatus = RecordStatusEnum.NotScanned,
	});

	factory FinishExamNotifyDetail.fromJson(Map<String, dynamic> map) {
		return FinishExamNotifyDetail( 
			recordCode: map['RecordCode'],
			recordStatus: RecordStatusEnum.values.firstWhere((e) => e.index == map['RecordStatus']),
		);
	}

	Map<String, dynamic> toJson() {
		final map = Map<String, dynamic>();
		if(recordCode != null)
			map['RecordCode'] = recordCode;
		map['RecordStatus'] = recordStatus.index;
		return map;
	}
}

class PushFinishExamNotifyToClientRequest {
	List<FinishExamNotifyDetail>? records;
	String? userCode;

	PushFinishExamNotifyToClientRequest({
		this.records,
		this.userCode,
	});

	factory PushFinishExamNotifyToClientRequest.fromJson(Map<String, dynamic> map) {
		return PushFinishExamNotifyToClientRequest( 
			records: map['Records'] != null ? (map['Records'] as List).map((e)=>FinishExamNotifyDetail.fromJson(e as Map<String,dynamic>)).toList() : null,
			userCode: map['UserCode'],
		);
	}

	Map<String, dynamic> toJson() {
		final map = Map<String, dynamic>();
		if(records != null)
			map['Records'] = records;
		if(userCode != null)
			map['UserCode'] = userCode;
		return map;
	}
}

class DeviceFinishExamRequest extends TokenRequest{
	List<String>? records;

	DeviceFinishExamRequest({
		this.records,
		String? token,
	}) : super(
			token: token,
		);

	factory DeviceFinishExamRequest.fromJson(Map<String, dynamic> map) {
		return DeviceFinishExamRequest( 
			records: map['Records'] != null ? map['Records'].cast<String>().toList() : null,
			token: map['Token'],
		);
	}

	Map<String, dynamic> toJson() {
		final map = super.toJson();
		if(records != null)
			map['Records'] = records;
		return map;
	}
}

class QueryExamListItemResult {
	String? examCode;
	String? patientName;
	String? age;
	String? sex;
	List<String>? associatedExamCodes;

	QueryExamListItemResult({
		this.examCode,
		this.patientName,
		this.age,
		this.sex,
		this.associatedExamCodes,
	});

	factory QueryExamListItemResult.fromJson(Map<String, dynamic> map) {
		return QueryExamListItemResult( 
			examCode: map['ExamCode'],
			patientName: map['PatientName'],
			age: map['Age'],
			sex: map['Sex'],
			associatedExamCodes: map['AssociatedExamCodes'] != null ? map['AssociatedExamCodes'].cast<String>().toList() : null,
		);
	}

	Map<String, dynamic> toJson() {
		final map = Map<String, dynamic>();
		if(examCode != null)
			map['ExamCode'] = examCode;
		if(patientName != null)
			map['PatientName'] = patientName;
		if(age != null)
			map['Age'] = age;
		if(sex != null)
			map['Sex'] = sex;
		if(associatedExamCodes != null)
			map['AssociatedExamCodes'] = associatedExamCodes;
		return map;
	}
}

enum ExamStatusEnum {
	Wait,
	Done,
}

class QueryExamListRequest extends PageRequest{
	String? patientName;
	DateTime? startTime;
	DateTime? endTime;
	ExamStatusEnum examStatus;

	QueryExamListRequest({
		this.patientName,
		this.startTime,
		this.endTime,
		this.examStatus = ExamStatusEnum.Wait,
		int pageIndex = 0,
		int pageSize = 0,
		String? token,
	}) : super(
			pageIndex: pageIndex,
			pageSize: pageSize,
			token: token,
		);

	factory QueryExamListRequest.fromJson(Map<String, dynamic> map) {
		return QueryExamListRequest( 
			patientName: map['PatientName'],
			startTime: map['StartTime'] != null ? DateTime.parse(map['StartTime']) : null,
			endTime: map['EndTime'] != null ? DateTime.parse(map['EndTime']) : null,
			examStatus: ExamStatusEnum.values.firstWhere((e) => e.index == map['ExamStatus']),
			pageIndex: map['PageIndex'],
			pageSize: map['PageSize'],
			token: map['Token'],
		);
	}

	Map<String, dynamic> toJson() {
		final map = super.toJson();
		if(patientName != null)
			map['PatientName'] = patientName;
		if(startTime != null)
			map['StartTime'] = JsonRpcUtils.dateFormat(startTime!);
		if(endTime != null)
			map['EndTime'] = JsonRpcUtils.dateFormat(endTime!);
		map['ExamStatus'] = examStatus.index;
		return map;
	}
}

class QueryExamInfoResult {
	DateTime? createTime;
	String? deptName;
	String? patientName;
	String? patientAge;
	String? patientSex;
	List<PatientInfoExt>? patientInfoExtList;
	List<String>? associatedExamCodes;

	QueryExamInfoResult({
		this.createTime,
		this.deptName,
		this.patientName,
		this.patientAge,
		this.patientSex,
		this.patientInfoExtList,
		this.associatedExamCodes,
	});

	factory QueryExamInfoResult.fromJson(Map<String, dynamic> map) {
		return QueryExamInfoResult( 
			createTime: map['CreateTime'] != null ? DateTime.parse(map['CreateTime']) : null,
			deptName: map['DeptName'],
			patientName: map['PatientName'],
			patientAge: map['PatientAge'],
			patientSex: map['PatientSex'],
			patientInfoExtList: map['PatientInfoExtList'] != null ? (map['PatientInfoExtList'] as List).map((e)=>PatientInfoExt.fromJson(e as Map<String,dynamic>)).toList() : null,
			associatedExamCodes: map['AssociatedExamCodes'] != null ? map['AssociatedExamCodes'].cast<String>().toList() : null,
		);
	}

	Map<String, dynamic> toJson() {
		final map = Map<String, dynamic>();
		if(createTime != null)
			map['CreateTime'] = JsonRpcUtils.dateFormat(createTime!);
		if(deptName != null)
			map['DeptName'] = deptName;
		if(patientName != null)
			map['PatientName'] = patientName;
		if(patientAge != null)
			map['PatientAge'] = patientAge;
		if(patientSex != null)
			map['PatientSex'] = patientSex;
		if(patientInfoExtList != null)
			map['PatientInfoExtList'] = patientInfoExtList;
		if(associatedExamCodes != null)
			map['AssociatedExamCodes'] = associatedExamCodes;
		return map;
	}
}

class QueryExamInfoRequest extends TokenRequest{
	String? examCode;

	QueryExamInfoRequest({
		this.examCode,
		String? token,
	}) : super(
			token: token,
		);

	factory QueryExamInfoRequest.fromJson(Map<String, dynamic> map) {
		return QueryExamInfoRequest( 
			examCode: map['ExamCode'],
			token: map['Token'],
		);
	}

	Map<String, dynamic> toJson() {
		final map = super.toJson();
		if(examCode != null)
			map['ExamCode'] = examCode;
		return map;
	}
}

enum Unit {
	None,
	percent,
	fraction,
	placeHolder_3,
	placeHolder_4,
	placeHolder_5,
	placeHolder_6,
	placeHolder_7,
	placeHolder_8,
	placeHolder_9,
	cm,
	mm,
	inch,
	ft,
	placeHolder_14,
	placeHolder_15,
	placeHolder_16,
	placeHolder_17,
	placeHolder_18,
	placeHolder_19,
	s,
	minute,
	hour,
	day,
	week,
	week_day,
	Tick,
	msec,
	placeHolder_28,
	placeHolder_29,
	degree,
	radian,
	placeHolder_32,
	placeHolder_33,
	placeHolder_34,
	placeHolder_35,
	placeHolder_36,
	placeHolder_37,
	placeHolder_38,
	placeHolder_39,
	g,
	mg,
	ng,
	kg,
	oz,
	lb,
	lb_oz,
	placeHolder_47,
	placeHolder_48,
	placeHolder_49,
	cm2,
	mm2,
	m2,
	placeHolder_53,
	placeHolder_54,
	placeHolder_55,
	placeHolder_56,
	placeHolder_57,
	placeHolder_58,
	placeHolder_59,
	cm3,
	mm3,
	ml,
	L,
	placeHolder_64,
	placeHolder_65,
	placeHolder_66,
	placeHolder_67,
	placeHolder_68,
	placeHolder_69,
	cms,
	mms,
	ms,
	placeHolder_73,
	placeHolder_74,
	placeHolder_75,
	placeHolder_76,
	placeHolder_77,
	placeHolder_78,
	placeHolder_79,
	cms2,
	mms2,
	placeHolder_82,
	placeHolder_83,
	placeHolder_84,
	placeHolder_85,
	placeHolder_86,
	placeHolder_87,
	placeHolder_88,
	placeHolder_89,
	cm3s,
	mls,
	mlmin,
	Lmin,
	placeHolder_94,
	placeHolder_95,
	placeHolder_96,
	placeHolder_97,
	placeHolder_98,
	placeHolder_99,
	gcm3,
	gml,
	ngml,
	placeHolder_103,
	placeHolder_104,
	placeHolder_105,
	placeHolder_106,
	placeHolder_107,
	placeHolder_108,
	placeHolder_109,
	mmHg,
	placeHolder_111,
	placeHolder_112,
	placeHolder_113,
	placeHolder_114,
	placeHolder_115,
	placeHolder_116,
	placeHolder_117,
	placeHolder_118,
	placeHolder_119,
	mV,
	placeHolder_121,
	placeHolder_122,
	placeHolder_123,
	placeHolder_124,
	placeHolder_125,
	placeHolder_126,
	placeHolder_127,
	placeHolder_128,
	placeHolder_129,
	Hz,
	KHz,
	HR,
	placeHolder_133,
	placeHolder_134,
	placeHolder_135,
	placeHolder_136,
	placeHolder_137,
	placeHolder_138,
	placeHolder_139,
	cm3m2,
	mlm2,
	placeHolder_142,
	placeHolder_143,
	placeHolder_144,
	placeHolder_145,
	placeHolder_146,
	placeHolder_147,
	placeHolder_148,
	placeHolder_149,
	cm3sm2,
	mlsm2,
	placeHolder_152,
	cm3minm2,
	mlminm2,
	Lminm2,
	placeHolder_156,
	placeHolder_157,
	placeHolder_158,
	placeHolder_159,
	circs,
	placeHolder_161,
	placeHolder_162,
	placeHolder_163,
	placeHolder_164,
	placeHolder_165,
	placeHolder_166,
	placeHolder_167,
	placeHolder_168,
	placeHolder_169,
	mlbeat,
	placeHolder_171,
	placeHolder_172,
	placeHolder_173,
	placeHolder_174,
	placeHolder_175,
	placeHolder_176,
	placeHolder_177,
	placeHolder_178,
	placeHolder_179,
	mm2pa,
	d1mpa,
	kpa,
	placeHolder_183,
	placeHolder_184,
	placeHolder_185,
	placeHolder_186,
	placeHolder_187,
	placeHolder_188,
	placeHolder_189,
	mmHgs,
	placeHolder_191,
	placeHolder_192,
	placeHolder_193,
	placeHolder_194,
	placeHolder_195,
	placeHolder_196,
	placeHolder_197,
	placeHolder_198,
	placeHolder_199,
	gm2,
	kgm2,
	placeHolder_202,
	placeHolder_203,
	placeHolder_204,
	placeHolder_205,
	placeHolder_206,
	placeHolder_207,
	placeHolder_208,
	placeHolder_209,
	cm2m2,
	placeHolder_211,
	placeHolder_212,
	placeHolder_213,
	placeHolder_214,
	placeHolder_215,
	placeHolder_216,
	placeHolder_217,
	placeHolder_218,
	placeHolder_219,
	cmm2,
	mmm2,
	placeHolder_222,
	placeHolder_223,
	placeHolder_224,
	placeHolder_225,
	placeHolder_226,
	placeHolder_227,
	placeHolder_228,
	placeHolder_229,
	pers,
	placeHolder_231,
	placeHolder_232,
	placeHolder_233,
	placeHolder_234,
	placeHolder_235,
	placeHolder_236,
	placeHolder_237,
	placeHolder_238,
	placeHolder_239,
	placeHolder_240,
	placeHolder_241,
	placeHolder_242,
	placeHolder_243,
	placeHolder_244,
	placeHolder_245,
	placeHolder_246,
	placeHolder_247,
	placeHolder_248,
	placeHolder_249,
	Celsius,
	Fahrenheit,
	placeHolder_252,
	placeHolder_253,
	placeHolder_254,
	placeHolder_255,
	placeHolder_256,
	placeHolder_257,
	placeHolder_258,
	placeHolder_259,
	Ohm,
	placeHolder_261,
	placeHolder_262,
	placeHolder_263,
	placeHolder_264,
	placeHolder_265,
	placeHolder_266,
	placeHolder_267,
	placeHolder_268,
	placeHolder_269,
	mmHgml,
}

class OutputItemMetaDTO {
	String? name;
	String? description;
	Unit unit;

	OutputItemMetaDTO({
		this.name,
		this.description,
		this.unit = Unit.None,
	});

	factory OutputItemMetaDTO.fromJson(Map<String, dynamic> map) {
		return OutputItemMetaDTO( 
			name: map['Name'],
			description: map['Description'],
			unit: Unit.values.firstWhere((e) => e.index == map['Unit']),
		);
	}

	Map<String, dynamic> toJson() {
		final map = Map<String, dynamic>();
		if(name != null)
			map['Name'] = name;
		if(description != null)
			map['Description'] = description;
		map['Unit'] = unit.index;
		return map;
	}
}

class CalculatorMetaDTO {
	List<OutputItemMetaDTO>? availableOutputs;

	CalculatorMetaDTO({
		this.availableOutputs,
	});

	factory CalculatorMetaDTO.fromJson(Map<String, dynamic> map) {
		return CalculatorMetaDTO( 
			availableOutputs: map['AvailableOutputs'] != null ? (map['AvailableOutputs'] as List).map((e)=>OutputItemMetaDTO.fromJson(e as Map<String,dynamic>)).toList() : null,
		);
	}

	Map<String, dynamic> toJson() {
		final map = Map<String, dynamic>();
		if(availableOutputs != null)
			map['AvailableOutputs'] = availableOutputs;
		return map;
	}
}

class ChildItemMetaDTO {
	String? name;
	String? description;
	bool isWorking;
	List<ChildItemMetaDTO>? childItems;
	CalculatorMetaDTO? calculator;

	ChildItemMetaDTO({
		this.name,
		this.description,
		this.isWorking = false,
		this.childItems,
		this.calculator,
	});

	factory ChildItemMetaDTO.fromJson(Map<String, dynamic> map) {
		return ChildItemMetaDTO( 
			name: map['Name'],
			description: map['Description'],
			isWorking: map['IsWorking'],
			childItems: map['ChildItems'] != null ? (map['ChildItems'] as List).map((e)=>ChildItemMetaDTO.fromJson(e as Map<String,dynamic>)).toList() : null,
			calculator: map['Calculator'] != null ? CalculatorMetaDTO.fromJson(map['Calculator']) : null,
		);
	}

	Map<String, dynamic> toJson() {
		final map = Map<String, dynamic>();
		if(name != null)
			map['Name'] = name;
		if(description != null)
			map['Description'] = description;
		map['IsWorking'] = isWorking;
		if(childItems != null)
			map['ChildItems'] = childItems;
		if(calculator != null)
			map['Calculator'] = calculator;
		return map;
	}
}

class ItemMetaDTO {
	String? name;
	String? description;
	String? briefAnnotation;
	String? measureTypeName;
	List<String>? categories;
	CalculatorMetaDTO? calculator;
	List<ChildItemMetaDTO>? multiMethodItems;

	ItemMetaDTO({
		this.name,
		this.description,
		this.briefAnnotation,
		this.measureTypeName,
		this.categories,
		this.calculator,
		this.multiMethodItems,
	});

	factory ItemMetaDTO.fromJson(Map<String, dynamic> map) {
		return ItemMetaDTO( 
			name: map['Name'],
			description: map['Description'],
			briefAnnotation: map['BriefAnnotation'],
			measureTypeName: map['MeasureTypeName'],
			categories: map['Categories'] != null ? map['Categories'].cast<String>().toList() : null,
			calculator: map['Calculator'] != null ? CalculatorMetaDTO.fromJson(map['Calculator']) : null,
			multiMethodItems: map['MultiMethodItems'] != null ? (map['MultiMethodItems'] as List).map((e)=>ChildItemMetaDTO.fromJson(e as Map<String,dynamic>)).toList() : null,
		);
	}

	Map<String, dynamic> toJson() {
		final map = Map<String, dynamic>();
		if(name != null)
			map['Name'] = name;
		if(description != null)
			map['Description'] = description;
		if(briefAnnotation != null)
			map['BriefAnnotation'] = briefAnnotation;
		if(measureTypeName != null)
			map['MeasureTypeName'] = measureTypeName;
		if(categories != null)
			map['Categories'] = categories;
		if(calculator != null)
			map['Calculator'] = calculator;
		if(multiMethodItems != null)
			map['MultiMethodItems'] = multiMethodItems;
		return map;
	}
}

class MeasureFolderDTO {
	String? name;
	List<String>? workingItemNames;
	List<ItemMetaDTO>? availableItems;

	MeasureFolderDTO({
		this.name,
		this.workingItemNames,
		this.availableItems,
	});

	factory MeasureFolderDTO.fromJson(Map<String, dynamic> map) {
		return MeasureFolderDTO( 
			name: map['Name'],
			workingItemNames: map['WorkingItemNames'] != null ? map['WorkingItemNames'].cast<String>().toList() : null,
			availableItems: map['AvailableItems'] != null ? (map['AvailableItems'] as List).map((e)=>ItemMetaDTO.fromJson(e as Map<String,dynamic>)).toList() : null,
		);
	}

	Map<String, dynamic> toJson() {
		final map = Map<String, dynamic>();
		if(name != null)
			map['Name'] = name;
		if(workingItemNames != null)
			map['WorkingItemNames'] = workingItemNames;
		if(availableItems != null)
			map['AvailableItems'] = availableItems;
		return map;
	}
}

class MeasureGroupDTO {
	String? name;
	String? description;
	List<MeasureFolderDTO>? availableFolders;

	MeasureGroupDTO({
		this.name,
		this.description,
		this.availableFolders,
	});

	factory MeasureGroupDTO.fromJson(Map<String, dynamic> map) {
		return MeasureGroupDTO( 
			name: map['Name'],
			description: map['Description'],
			availableFolders: map['AvailableFolders'] != null ? (map['AvailableFolders'] as List).map((e)=>MeasureFolderDTO.fromJson(e as Map<String,dynamic>)).toList() : null,
		);
	}

	Map<String, dynamic> toJson() {
		final map = Map<String, dynamic>();
		if(name != null)
			map['Name'] = name;
		if(description != null)
			map['Description'] = description;
		if(availableFolders != null)
			map['AvailableFolders'] = availableFolders;
		return map;
	}
}

class MeasureModeDTO {
	String? modeName;
	List<MeasureGroupDTO>? availableGroups;

	MeasureModeDTO({
		this.modeName,
		this.availableGroups,
	});

	factory MeasureModeDTO.fromJson(Map<String, dynamic> map) {
		return MeasureModeDTO( 
			modeName: map['ModeName'],
			availableGroups: map['AvailableGroups'] != null ? (map['AvailableGroups'] as List).map((e)=>MeasureGroupDTO.fromJson(e as Map<String,dynamic>)).toList() : null,
		);
	}

	Map<String, dynamic> toJson() {
		final map = Map<String, dynamic>();
		if(modeName != null)
			map['ModeName'] = modeName;
		if(availableGroups != null)
			map['AvailableGroups'] = availableGroups;
		return map;
	}
}

class MeasureApplicationDTO {
	String? id;
	String? description;
	List<MeasureModeDTO>? availableModes;

	MeasureApplicationDTO({
		this.id,
		this.description,
		this.availableModes,
	});

	factory MeasureApplicationDTO.fromJson(Map<String, dynamic> map) {
		return MeasureApplicationDTO( 
			id: map['Id'],
			description: map['Description'],
			availableModes: map['AvailableModes'] != null ? (map['AvailableModes'] as List).map((e)=>MeasureModeDTO.fromJson(e as Map<String,dynamic>)).toList() : null,
		);
	}

	Map<String, dynamic> toJson() {
		final map = Map<String, dynamic>();
		if(id != null)
			map['Id'] = id;
		if(description != null)
			map['Description'] = description;
		if(availableModes != null)
			map['AvailableModes'] = availableModes;
		return map;
	}
}

class GetMeasureApplicationRequest extends TokenRequest{
	String? applicationName;
	String? categoryName;
	List<String>? measureModes;

	GetMeasureApplicationRequest({
		this.applicationName,
		this.categoryName,
		this.measureModes,
		String? token,
	}) : super(
			token: token,
		);

	factory GetMeasureApplicationRequest.fromJson(Map<String, dynamic> map) {
		return GetMeasureApplicationRequest( 
			applicationName: map['ApplicationName'],
			categoryName: map['CategoryName'],
			measureModes: map['MeasureModes'] != null ? map['MeasureModes'].cast<String>().toList() : null,
			token: map['Token'],
		);
	}

	Map<String, dynamic> toJson() {
		final map = super.toJson();
		if(applicationName != null)
			map['ApplicationName'] = applicationName;
		if(categoryName != null)
			map['CategoryName'] = categoryName;
		if(measureModes != null)
			map['MeasureModes'] = measureModes;
		return map;
	}
}

class CommentItemDTO {
	String? text;

	CommentItemDTO({
		this.text,
	});

	factory CommentItemDTO.fromJson(Map<String, dynamic> map) {
		return CommentItemDTO( 
			text: map['Text'],
		);
	}

	Map<String, dynamic> toJson() {
		final map = Map<String, dynamic>();
		if(text != null)
			map['Text'] = text;
		return map;
	}
}

class GetCommentsByApplicationRequest extends TokenRequest{
	String? languageCode;
	String? applicationName;
	String? categoryName;

	GetCommentsByApplicationRequest({
		this.languageCode,
		this.applicationName,
		this.categoryName,
		String? token,
	}) : super(
			token: token,
		);

	factory GetCommentsByApplicationRequest.fromJson(Map<String, dynamic> map) {
		return GetCommentsByApplicationRequest( 
			languageCode: map['LanguageCode'],
			applicationName: map['ApplicationName'],
			categoryName: map['CategoryName'],
			token: map['Token'],
		);
	}

	Map<String, dynamic> toJson() {
		final map = super.toJson();
		if(languageCode != null)
			map['LanguageCode'] = languageCode;
		if(applicationName != null)
			map['ApplicationName'] = applicationName;
		if(categoryName != null)
			map['CategoryName'] = categoryName;
		return map;
	}
}

class PresetCommentItemDTO {
	List<String>? categoryList;
	String? text;

	PresetCommentItemDTO({
		this.categoryList,
		this.text,
	});

	factory PresetCommentItemDTO.fromJson(Map<String, dynamic> map) {
		return PresetCommentItemDTO( 
			categoryList: map['CategoryList'] != null ? map['CategoryList'].cast<String>().toList() : null,
			text: map['Text'],
		);
	}

	Map<String, dynamic> toJson() {
		final map = Map<String, dynamic>();
		if(categoryList != null)
			map['CategoryList'] = categoryList;
		if(text != null)
			map['Text'] = text;
		return map;
	}
}

class GetPresetCommentsRequest extends TokenRequest{
	String? languageCode;

	GetPresetCommentsRequest({
		this.languageCode,
		String? token,
	}) : super(
			token: token,
		);

	factory GetPresetCommentsRequest.fromJson(Map<String, dynamic> map) {
		return GetPresetCommentsRequest( 
			languageCode: map['LanguageCode'],
			token: map['Token'],
		);
	}

	Map<String, dynamic> toJson() {
		final map = super.toJson();
		if(languageCode != null)
			map['LanguageCode'] = languageCode;
		return map;
	}
}

class DeviceDiagnosisChangedNotification {
	String? deviceCode;

	DeviceDiagnosisChangedNotification({
		this.deviceCode,
	});

	factory DeviceDiagnosisChangedNotification.fromJson(Map<String, dynamic> map) {
		return DeviceDiagnosisChangedNotification( 
			deviceCode: map['DeviceCode'],
		);
	}

	Map<String, dynamic> toJson() {
		final map = Map<String, dynamic>();
		if(deviceCode != null)
			map['DeviceCode'] = deviceCode;
		return map;
	}
}


